package com.liquidnet.service.kylin.service.impl.partner;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.liquidnet.commons.lang.util.IDGenerator;
import com.liquidnet.service.kylin.dto.param.PerformanceCreateParam;
import com.liquidnet.service.kylin.dto.param.PerformanceStep1Param;
import com.liquidnet.service.kylin.dto.param.PerformanceStep2Param;
import com.liquidnet.service.kylin.dto.vo.partner.TicketCreatePartnerVo;
import com.liquidnet.service.kylin.dto.vo.partner.TicketTimesCreatePartnerVo;
import com.liquidnet.service.kylin.dto.vo.partner.TicketTimesTicketCreatePartnerVo;
import com.liquidnet.service.kylin.entity.KylinPerformances;
import com.liquidnet.service.kylin.mapper.KylinPerformancesMapper;
import com.liquidnet.service.kylin.service.partner.IKylinPerformancesPartnerService;
import com.mongodb.BasicDBObject;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.convert.MongoConverter;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * <p>
 * 演出 服务实现类
 * </p>
 *
 * @author liquidnet
 * @since 2021-05-05
 */
@Service
public class KylinPerformancesPartnerServiceImpl extends ServiceImpl<KylinPerformancesMapper, KylinPerformances> implements IKylinPerformancesPartnerService {

    @Autowired
    private MongoTemplate mongoTemplate;

    @Autowired
    private MongoConverter mongoConverter;

    @Autowired
    private KylinTicketsPartnerServiceImpl ticketsPartnerService;

    @Autowired
    private KylinTicketTimesPartnerServiceImpl ticketTimesPartnerService;

    @Override
    public String createStep1(PerformanceStep1Param step1Param) {
        // 获取 主键id
        String performanceId = IDGenerator.nextSnowId().toString();
        // 获取 当前时间 -> 创建时间
        LocalDateTime createdAt = LocalDateTime.now();

        try {
            step1Param.setCreatedAt(createdAt.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
            step1Param.setPerformancesId(performanceId);
            PerformanceStep1Param data = mongoTemplate.insert(
                    step1Param, PerformanceStep1Param.class.getSimpleName()
            );
            return performanceId;
        } catch (Exception e) {
            return "";
        }
    }

    @Override
    public boolean createStep2(PerformanceStep2Param step2Param) {

        // 获取 主键id
        String performanceId = step2Param.getPerformancesId();
        // 获取 当前时间 -> 创建时间
        LocalDateTime createdAt = LocalDateTime.now();

        PerformanceStep1Param info = mongoTemplate.findOne(Query.query(Criteria.where("performancesId").is(performanceId)),
                PerformanceStep1Param.class,
                PerformanceStep1Param.class.getSimpleName());

        boolean isExists = mongoTemplate.exists(Query.query(Criteria.where("performancesId").is(performanceId).and("status").is(0)),
                PerformanceCreateParam.class,
                PerformanceCreateParam.class.getSimpleName());

        boolean isExistsSubmit = mongoTemplate.exists(Query.query(Criteria.where("performancesId").is(performanceId).and("status").is(1)),
                PerformanceCreateParam.class,
                PerformanceCreateParam.class.getSimpleName());

        if (isExistsSubmit) {
            return false;
        }

        //演出修改数据
        HashMap<String, Object> map = new HashMap<>();
        map.put("isTrueName", step2Param.getIsTrueName());
        map.put("limitCount", step2Param.getLimitCount());
        map.put("createdAt", createdAt.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        map.put("isSubmit", step2Param.getIsSubmit());
        map.put("merchantId", step2Param.getMerchantId());

        //演出新增数据
        PerformanceCreateParam performanceCreateParam = new PerformanceCreateParam();
        BeanUtils.copyProperties(info, performanceCreateParam);
        performanceCreateParam.setIsTrueName(step2Param.getIsTrueName());
        performanceCreateParam.setLimitCount(step2Param.getLimitCount());
        performanceCreateParam.setCreatedAt(createdAt.format(DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss")));
        performanceCreateParam.setIsSubmit(step2Param.getIsSubmit());
        performanceCreateParam.setMerchantId(step2Param.getMerchantId());

        // 获取第一步数据
        if (step2Param.getIsSubmit() == 1) { // 提交
            if (isExists) {
                map.put("status", 1);
                BasicDBObject object = new BasicDBObject("$set", mongoConverter.convertToMongoType(map));
                mongoTemplate.getCollection(PerformanceCreateParam.class.getSimpleName()).updateOne(
                        Query.query(Criteria.where("performancesId").is(performanceId)).getQueryObject(),
                        object
                );
            }else{
                // 提交 演出
                performanceCreateParam.setStatus(1);
                PerformanceCreateParam data = mongoTemplate.insert(
                        performanceCreateParam, PerformanceCreateParam.class.getSimpleName()
                );
            }


            //提交 票
            List<TicketTimesCreatePartnerVo> ticketTimesCreatePartnerVoList = ticketTimesPartnerService.getTimesMongo(performanceId);
            for (TicketTimesCreatePartnerVo ticketTimes : ticketTimesCreatePartnerVoList) {
                ticketsPartnerService.submitTicketByTimes(ticketTimes.getTicketTimesId());
            }
        } else { //保存
            if (isExists) {
                map.put("status", 0);
                BasicDBObject object = new BasicDBObject("$set", mongoConverter.convertToMongoType(map));
                mongoTemplate.getCollection(PerformanceCreateParam.class.getSimpleName()).updateMany(
                        Query.query(Criteria.where("performancesId").is(performanceId)).getQueryObject(),
                        object
                );
            }else {
                // 保存 演出
                performanceCreateParam.setStatus(0);
                PerformanceCreateParam data = mongoTemplate.insert(
                        performanceCreateParam, PerformanceCreateParam.class.getSimpleName()
                );
            }
        }

        return true;
    }

    @Override
    public PerformanceStep1Param getStep1(String performancesId) {
        PerformanceStep1Param data = mongoTemplate.findOne(
                Query.query(Criteria.where("performancesId").is(performancesId)), PerformanceStep1Param.class,PerformanceStep1Param.class.getSimpleName()
        );
        if(data==null){
            return new PerformanceStep1Param();
        }
        return data;
    }

    @Override
    public PerformanceStep2Param getStep2(String performancesId) {
        PerformanceCreateParam data = mongoTemplate.findOne(
                Query.query(Criteria.where("performancesId").is(performancesId)), PerformanceCreateParam.class,PerformanceCreateParam.class.getSimpleName()
        );

        if(data==null){
            return new PerformanceStep2Param();
        }

        List<TicketTimesCreatePartnerVo> ticketTimesCreatePartnerVos = ticketTimesPartnerService.getTimesMongo(performancesId);
        List<TicketTimesTicketCreatePartnerVo> ticketTimesTicketCreatePartnerVoList = new ArrayList<TicketTimesTicketCreatePartnerVo>();

        for (TicketTimesCreatePartnerVo ticketTimes : ticketTimesCreatePartnerVos) {
            TicketTimesTicketCreatePartnerVo ticketTimesTicketCreatePartnerVo = new TicketTimesTicketCreatePartnerVo();
            BeanUtils.copyProperties(ticketTimes, ticketTimesTicketCreatePartnerVo);
            List<TicketCreatePartnerVo> ticketCreatePartnerVos = ticketsPartnerService.getTicketMongoList(ticketTimes.getTicketTimesId());
            ticketTimesTicketCreatePartnerVo.setTicket(ticketCreatePartnerVos);
            ticketTimesTicketCreatePartnerVoList.add(ticketTimesTicketCreatePartnerVo);
        }

        PerformanceStep2Param performanceStep2Param = new PerformanceStep2Param();
        BeanUtils.copyProperties(data,performanceStep2Param);
        performanceStep2Param.setTicketTimes(ticketTimesTicketCreatePartnerVoList);

        return performanceStep2Param;
    }
}
